//***********************************************************************/
//*                                                                     */
//*     IF-70@API_Test  for Visual C#2005                              */
//*       Interface  USB / RS-232C                                      */
//*                                                                     */
//***********************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;   // for DllImport

namespace PSA_Control
{
    public partial class Form1 : Form
    {
        // IF-60 static value
        private int pc_addr  = 1;                // PSA PC-ADDRESS
        private int sys_addr = 1;                // PSA SYSTEM-ADDRESS
        private int com_if   = -1;               // 0:USB  1:NI-GPIB  2:RS-232C  -1:Offline


        // DLL Import for IF-60
        [DllImport("if_60.dll")] static extern int USB488_DeviceOpen(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_DeviceClose(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_SetTimeout(int addrs, int m);
        [DllImport("if_60.dll")] static extern int USB488_SetRemote(int addrs, int m);
        [DllImport("if_60.dll")] static extern int USB488_GetRemote(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_DevClear(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_GetSTBQuery(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_ClearSTB(int addrs);
        [DllImport("if_60.dll")] static extern int USB488_Send(int addrs, string p);
        [DllImport("if_60.dll")] static extern int USB488_Receive(int addrs, int cnt, StringBuilder p);

        // Form Init
        public Form1()
        {
            InitializeComponent();

            // Initialize the Combo1 with Interface
            Combo1.Items.Add("USB");
            Combo1.Items.Add("COM");

            Combo1.SelectedIndex = 0;

            // Initialize the ComboBox2 with the PC Address possible values
            for (int i = 1; i <= 30; i++)
            {
                Combo2.Items.Add(i);
            }
            Combo2.SelectedIndex = 0;

            // Initialize the ComboBox3 with the System Address possible values
            for (int i = 1; i <= 30; i++)
            {
                Combo3.Items.Add(i);
            }
            Combo3.SelectedIndex = 0;
        }

        // Form Close (x Button)
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            int retn;
            if (com_if >= 0)
            {
                retn = PSA_Send(com_if, pc_addr, "OUTPUT 0");
                retn = PSA_Send(com_if, pc_addr, "ADRS 1");
                retn = PSA_Send(com_if, pc_addr, "GTL"); 
                retn = PSA_DeviceClose(com_if, pc_addr);
            }
        }
        
        // Click Open button 
        private void Open_Button_Click(object sender, EventArgs e)
        {
            int retn;                       // Return Code
            int Deviceid;

            // Get Addr&IF
            com_if = Combo1.SelectedIndex;
            pc_addr = Combo2.SelectedIndex + 1;
            sys_addr = Combo3.SelectedIndex + 1;

            if (Open_Button.Text == "OPEN")
            {
                // Device Open
                Deviceid = PSA_DeviceOpen(com_if, pc_addr);
                if (Deviceid > 0)                               //Open OK
                {
                    Open_Button.Text    = "CLOSE";
                    Output_Button.Enabled = true;
                    Pre1_Button.Enabled = true;
                    Pre2_Button.Enabled = true;
                    Pre3_Button.Enabled = true;
                    //PreOFF_Button.Enabled = true;
                    Read_Button.Enabled = true;
                    Combo1.Enabled      = false;
                    Combo2.Enabled      = false;
                    Combo3.Enabled      = false;
                    Text1.Text          = "";
                }
                else
                {
                    Text1.Text = "Device Open Error";
                    com_if = -1;
                }

            }
            else
            {
                // Device Close
                Output_Button.Text = "ON";
                Output_Button.BackColor = System.Drawing.SystemColors.Control;
                Open_Button.Text = "OPEN";
                Output_Button.Enabled   = false ;
                Pre1_Button.Enabled     = false ;
                Pre2_Button.Enabled     = false ;
                Pre3_Button.Enabled     = false ;
                PreOFF_Button.Enabled   = false ;
                Read_Button.Enabled     = false ;
                Combo1.Enabled          = true ;
                Combo2.Enabled          = true ;
                Combo3.Enabled          = true ;
                Text1.Text = "";
                retn = PSA_Send(com_if, pc_addr, "OUTPUT 0");
                retn = PSA_Send(com_if, pc_addr, "ADRS 1");
                retn = PSA_Send(com_if, pc_addr, "GTL");
                retn = PSA_DeviceClose(com_if, pc_addr);
                com_if = -1;
            }
        }

        //Click OUT Button
        private void Output_Button_Click(object sender, EventArgs e)
        {
            int retn;
            if (Output_Button.Text == "ON")
            {
                Output_Button.Text = "OFF";
                Output_Button.BackColor = System.Drawing.Color.Red;
                retn = PSA_Send(com_if, pc_addr, "OUTPUT 1");
            }
            else
            {
                Output_Button.Text = "ON";
                Output_Button.BackColor = System.Drawing.SystemColors.Control;
                retn = PSA_Send(com_if, pc_addr, "OUTPUT 0");
            }
        }

        //Click Preset1 Button
        private void Pre1_Button_Click(object sender, EventArgs e)
        {
            int retn;

            retn = PSA_Send(com_if, pc_addr, "PRESET 1");
        }

        //Click Preset2 Button
        private void Pre2_Button_Click(object sender, EventArgs e)
        {
            int retn;

            retn = PSA_Send(com_if, pc_addr, "PRESET 2");
        }

        //Click Preset3 Button
        private void Pre3_Button_Click(object sender, EventArgs e)
        {
            int retn;

            retn = PSA_Send(com_if, pc_addr, "PRESET 3");
        }

        //Click Preset OFF Button
        private void PreOFF_Button_Click(object sender, EventArgs e)
        {
            int retn;

            retn = PSA_Send(com_if, pc_addr, "PRESET 1");
        }

        //Click Read Button        
        private void Read_Button_Click(object sender, EventArgs e)
        {
            int retn;
            string bbuf;
            

            retn = PSA_Send(com_if, pc_addr, "STATUS?");
            bbuf = PSA_Read(com_if , pc_addr, 64);
            string[] mbuf = bbuf.Split(',');
            Text1.Text = mbuf[2] + " V" + System.Environment.NewLine + mbuf[3] + " A";
        }

        // Display Change
        private void Combo1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (Combo1.Text == "COM")
            {
                Label4.Text = "COM Port";
            }
            else
            {
                Label4.Text = "PC ADDR";
            }
        }

        // IF-60/70 Device Open        
        private int PSA_DeviceOpen(int comif, int  addr)
        {
            int hHandle;
            int retn;
            byte adrs;

            adrs = (byte)addr;
            hHandle = -1;
            switch (comif)
            {
                case 0: //USB
                    try
                    {
                        hHandle = USB488_DeviceOpen(addr);
                    }
                    catch (System.Exception caught)
                    {
                        MessageBox.Show(caught.Message, "Open Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break; 
                    }

                    retn = USB488_SetTimeout(addr, 3000);
                    break;

                case 1: //RS-232C
                    serialPort1.PortName  = "COM" + pc_addr.ToString();
                    serialPort1.BaudRate  = 9600;
                    serialPort1.DataBits  = 7;
                    serialPort1.Parity    = System.IO.Ports.Parity.Even ;
                    serialPort1.StopBits  = System.IO.Ports.StopBits.One;
                    serialPort1.Handshake = System.IO.Ports.Handshake.None ;
                    serialPort1.RtsEnable = true;
                    serialPort1.ReadTimeout = 3000;
                    serialPort1.WriteTimeout = 500;
                    serialPort1.Open();
                    serialPort1.DiscardInBuffer();
                    hHandle = pc_addr;
                    break;

                default:
                    hHandle = -1;
                    break;
            }
            if (hHandle > 0)
            {
                retn = PSA_Send(comif, addr, "ADRS " + sys_addr.ToString());
                if (retn >= 0)
                {
                    retn = PSA_Send(comif, addr, "OUTPUT 0");
                    retn = PSA_Send(comif, addr, "MONDSP 0");
                }
                else
                {
                    hHandle = -1;
                    PSA_DeviceClose(comif, addr);
                }
            }
            return hHandle;
        }

        // IF-60/70 Device Close
        private int PSA_DeviceClose(int comif, int addr)
        {
            int retn;

            switch (comif)
            {
                case 0: //USB
                    retn = USB488_SetRemote(addr, 0);
                    retn = USB488_DeviceClose(addr);
                    break;

                case 1: //RS-232C
                    serialPort1.Close();
                    retn = 0;
                    break;

                default:
                    retn = -1;
                    break;
            }
            return retn;
        }

        // IF-60/70 Device Send Msg
        private int PSA_Send(int comif, int addr, string msg)
        {
            int retn;

            if ((msg.Length > 256)|| (msg.Length == 0)) return -2;

            retn = -1;
            try
            {
                switch (comif)
                {
                    case 0: //USB
                        retn = USB488_Send(addr, msg );
                        break;
                    
                    case 1: //RS-232C
                        serialPort1.DiscardInBuffer();
                        serialPort1.WriteLine(msg + "\n");
                        retn = 0;
                        break;

                    default:
                        break;
                }
            }
            catch (System.Exception caught)
            {
                MessageBox.Show(caught.Message, "SendErr", MessageBoxButtons.OK, MessageBoxIcon.Error);
                retn = -1;
            }
            return retn;
        }

        // IF-60/70 Device Read Msg
        private string PSA_Read(int comif, int addr, int cnt)
        {
            int lStatus = -1;
            string tmpstr = "";
            StringBuilder rstr = new StringBuilder(256);

            try
            {
                switch (comif)
                {
                    case 0: //USB
                        lStatus = USB488_Receive(addr, cnt, rstr );
                        tmpstr = rstr.ToString();
                        break;

                    case 1: //RS-232C
                        System.Threading.Thread.Sleep(200);
                        tmpstr = serialPort1.ReadExisting() ;
                        lStatus = tmpstr.Length;
                        break;

                    default :
                        break;
                }
                if (lStatus <= 1)
                {
                    MessageBox.Show("Read Error", "Read Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    tmpstr = "";
                }
            }
            catch (System.Exception caught)
            {
                MessageBox.Show(caught.Message, "Read Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                tmpstr = "";
            }
            return tmpstr;
        }
    }
}